const { response } = require("express");
const ProductModel = require("../model/ProductModel");

//CRUD Operations
const getBasicProductsData = async (req, res) => {
  try {
    const products = await ProductModel.find({ featured: true })
      .select({
        title: 1,
        price: 1,
        discount: 1,
        images: 1,
        slug: 1,
      })
      .limit(12);
    res.status(200).json(products);
  } catch (err) {
    res.status(400).send("Error Ocear \nError is:\n" + err);
  }
};

//Get All Products in specific Category
const getCategoryProducts = async (req, res) => {
  try {
    const products = await ProductModel.find({
      category: req.params.name,
    })
      .select({
        title: 1,
        price: 1,
        discount: 1,
        images: 1,
        slug: 1,
      })
      .sort({ title: 1, featured: 1 });
    res.status(200).json(products);
  } catch (err) {
    res.status(400).send("Error Ocear \nError is:\n" + err);
  }
};
const searchProducts = async (req, res) => {
  const products = await ProductModel.find({
    title: { $regex: req.params.name, $options: "i" },
  })
    .select({
      title: 1,
      price: 1,
      discount: 1,
      images: 1,
      slug: 1,
    })
    .sort({ title: 1, featured: 1 });
  res.json(products);
};

//Get All Products Slug
const productsSlug = async (req, res) => {
  const products = await ProductModel.find({}).select({
    _id: 0,
    slug: 1,
    category: 1,
  });
  res.status(200).json(products);
};

const getProductBySlug = async (req, res) => {
  const product = await ProductModel.findOne({ slug: req.params.slug });
  res.status(200).json(product);
};

const addProduct = (req, res) => {
  console.log(req.body);
  let product = new ProductModel(req.body);
  product
    .save()
    .then(() => res.send("Product Added"))
    .catch((err) =>
      res.status(400).send("Please provide a correct data, \nError is:\n" + err)
    );
};

const updateProduct = (req, res) => {
  ProductModel.findOneAndUpdate({ slug: req.params.slug }, req.body, {
    new: true,
  })
    .then((response) => {
      console.log("updated");
      res.json(response);
    })
    .catch((err) => res.json(err));
};
const deleteProduct = (req, res) => {
  ProductModel.findOneAndDelete({ slug: req.params.slug })
    .then((response) => {
      console.log(response);
      res.json({ message: response });
    })
    .catch((err) => res.json({ message: err }));
};

module.exports = {
  getProductBySlug,
  addProduct,
  updateProduct,
  deleteProduct,
  searchProducts,
  getBasicProductsData,
  getCategoryProducts,
  productsSlug,
};
